"""abdds 数据模型"""

from __future__ import annotations

from dataclasses import dataclass


@dataclass
class UploadResult:
    """上传结果"""

    fs_id: int | None
    md5: str
    path: str
    size: int | None


@dataclass
class _ApiPreUploadResponse:
    """预上传响应（内部模型）"""

    path: str
    uploadid: str
    block_list: list[int]
    block_list_md5: list[str]
    file_size: int


@dataclass
class ApiFileMeta:
    """文件元信息"""

    md5: str
    path: str
    size: int
    fs_id: int
    dlink: str
    filename: str


@dataclass
class ApiFileListItem:
    """文件列表项"""

    md5: str
    path: str
    size: int
    fs_id: int
    is_dir: int
    filename: str


@dataclass
class ApiQuotaInfo:
    """空间配额信息"""

    total: int
    used: int



"""上传相关逻辑 - 异步小文件单步上传 & 大文件分片上传"""

from __future__ import annotations

import hashlib
import inspect
import io
import json
import logging
from collections.abc import AsyncGenerator as _AsyncGeneratorABC
from collections.abc import Generator as _GeneratorABC
from pathlib import Path, PurePosixPath
from typing import TYPE_CHECKING, AsyncGenerator, Generator

import aiofiles
import aiofiles.os

from .errors import BaiduPanAPIError
from .models import _ApiPreUploadResponse, UploadResult

if TYPE_CHECKING:
    from ._http import _HttpTransport

logger = logging.getLogger("abdds")

BLOCK_SIZE = 4 * 1024 * 1024  # 4MB

# file_from 参数可接受类型
UploadSource = Path | bytes | Generator[bytes, None, None] | AsyncGenerator[bytes, None]


def _is_generator(obj: object) -> bool:
    """判断对象是否为同步生成器"""
    return isinstance(obj, _GeneratorABC) and not isinstance(obj, _AsyncGeneratorABC)


def _is_async_generator(obj: object) -> bool:
    """判断对象是否为异步生成器"""
    return isinstance(obj, _AsyncGeneratorABC)


async def upload(
    transport: _HttpTransport,
    file_from: UploadSource,
    to: PurePosixPath,
    pan_dir_base: str,
) -> UploadResult:
    """
    异步上传数据到百度网盘

    - 小文件 (<=4MB): 单步上传 (precreate + direct upload)
    - 大文件 (>4MB):  分片上传 (precreate + 分片 + merge)

    Args:
        transport: 异步 HTTP 传输层
        file_from: 数据来源，支持 Path / bytes / Generator[bytes] / AsyncGenerator[bytes]
        to: 远程路径 (PurePosixPath)，将拼接在 /apps/{app_name} 之后
        pan_dir_base: 应用目录前缀，如 /apps/myapp

    Raises:
        TypeError: file_from 类型不正确
    """
    # 类型校验
    if (
        not isinstance(file_from, (Path, bytes))
        and not _is_generator(file_from)
        and not _is_async_generator(file_from)
    ):
        raise TypeError(
            f"file_from must be Path, bytes, Generator[bytes], or AsyncGenerator[bytes], "
            f"got {type(file_from).__name__}"
        )

    # 构造完整远程路径: pan_dir_base + to
    remote_path = f"{pan_dir_base}{to.as_posix()}"

    # 根据 file_from 类型提取数据
    if isinstance(file_from, Path):
        if not file_from.is_file():
            raise FileNotFoundError(f"File not found: {file_from}")
        file_size = (await aiofiles.os.stat(file_from)).st_size
        if file_size == 0:
            raise ValueError("Cannot upload empty file (0 bytes)")
        block_md5_list = await _calculate_file_md5_blocks(file_from)
    elif isinstance(file_from, bytes):
        file_size = len(file_from)
        if file_size == 0:
            raise ValueError("Cannot upload empty data (0 bytes)")
        block_md5_list = _calculate_bytes_md5_blocks(file_from)
    elif _is_async_generator(file_from):
        # 异步生成器需要消费一次来计算大小和 MD5，缓存在内存中
        chunks: list[bytes] = []
        file_size = 0
        md5_list: list[str] = []
        current_chunk = io.BytesIO()

        async for data in file_from:
            current_chunk.write(data)
            file_size += len(data)
            if current_chunk.tell() >= BLOCK_SIZE:
                block_data = current_chunk.getvalue()
                chunks.append(block_data)
                md5_list.append(hashlib.md5(block_data).hexdigest())
                current_chunk = io.BytesIO()

        # 处理最后不足一个 BLOCK_SIZE 的数据
        remaining = current_chunk.getvalue()
        if remaining:
            chunks.append(remaining)
            md5_list.append(hashlib.md5(remaining).hexdigest())

        block_md5_list = md5_list

        if file_size == 0:
            raise ValueError("Cannot upload empty async generator (0 bytes yielded)")

        # 将 chunks 重新组装为异步生成器供上传使用
        async def _replay_chunks() -> AsyncGenerator[bytes, None]:
            for chunk in chunks:
                yield chunk

        file_from = _replay_chunks()
    elif _is_generator(file_from):
        # 同步生成器
        chunks2: list[bytes] = []
        file_size = 0
        md5_list2: list[str] = []
        current_chunk2 = io.BytesIO()

        for data in file_from:
            current_chunk2.write(data)
            file_size += len(data)
            if current_chunk2.tell() >= BLOCK_SIZE:
                block_data = current_chunk2.getvalue()
                chunks2.append(block_data)
                md5_list2.append(hashlib.md5(block_data).hexdigest())
                current_chunk2 = io.BytesIO()

        remaining2 = current_chunk2.getvalue()
        if remaining2:
            chunks2.append(remaining2)
            md5_list2.append(hashlib.md5(remaining2).hexdigest())

        block_md5_list = md5_list2

        if file_size == 0:
            raise ValueError("Cannot upload empty generator (0 bytes yielded)")

        file_from = (chunk for chunk in chunks2)
    else:
        raise TypeError(
            f"file_from must be Path, bytes, Generator[bytes], or AsyncGenerator[bytes], "
            f"got {type(file_from).__name__}"
        )

    logger.info("Uploading -> %s (%d bytes)", remote_path, file_size)

    # 预上传
    pre_resp = await _api_pre_create(transport, remote_path, file_size, block_md5_list)

    # 获取上传服务器
    upload_host = await _get_upload_server(transport, remote_path, pre_resp.uploadid)

    if file_size <= BLOCK_SIZE:
        return await _upload_small(transport, upload_host, pre_resp, file_from, file_size)
    else:
        return await _upload_multipart(transport, upload_host, pre_resp, file_from, block_md5_list)


async def _upload_small(
    transport: _HttpTransport,
    host: str,
    pre_resp: _ApiPreUploadResponse,
    file_from: UploadSource,
    file_size: int,
) -> UploadResult:
    """小文件单步上传"""
    url = f"{host}/rest/2.0/pcs/file"
    params = {"method": "upload", "path": pre_resp.path}

    if isinstance(file_from, Path):
        async with aiofiles.open(file_from, "rb") as f:
            data = await f.read()
    elif isinstance(file_from, bytes):
        data = file_from
    elif _is_async_generator(file_from):
        chunks: list[bytes] = []
        async for chunk in file_from:
            chunks.append(chunk)
        data = b"".join(chunks)
    elif _is_generator(file_from):
        data = b"".join(file_from)
    else:
        data = b""

    files = {"file": ("blob", data)}
    resp = await transport.request("POST", url, params=params, files=files)
    return UploadResult(
        fs_id=resp.get("fs_id"),
        md5=resp.get("md5"),
        path=resp.get("path"),
        size=resp.get("size"),
    )


async def _upload_multipart(
    transport: _HttpTransport,
    host: str,
    pre_resp: _ApiPreUploadResponse,
    file_from: UploadSource,
    block_md5_list: list[str],
) -> UploadResult:
    """大文件分片上传"""
    url_prefix = f"{host}/rest/2.0/pcs/superfile2"

    if isinstance(file_from, Path):
        async with aiofiles.open(file_from, "rb") as f:
            for idx in range(len(block_md5_list)):
                await f.seek(idx * BLOCK_SIZE)
                chunk_data = await f.read(BLOCK_SIZE)
                await _upload_part(transport, url_prefix, pre_resp, idx, chunk_data)
    elif isinstance(file_from, bytes):
        for idx in range(len(block_md5_list)):
            chunk_data = file_from[idx * BLOCK_SIZE : (idx + 1) * BLOCK_SIZE]
            await _upload_part(transport, url_prefix, pre_resp, idx, chunk_data)
    elif _is_async_generator(file_from):
        idx = 0
        async for chunk_data in file_from:
            await _upload_part(transport, url_prefix, pre_resp, idx, chunk_data)
            idx += 1
    elif _is_generator(file_from):
        for idx, chunk_data in enumerate(file_from):
            await _upload_part(transport, url_prefix, pre_resp, idx, chunk_data)

    # 合并分片
    return await _api_create_file(
        transport, pre_resp.path, pre_resp.uploadid, pre_resp.file_size, block_md5_list
    )


async def _upload_part(
    transport: _HttpTransport,
    url_prefix: str,
    pre_resp: _ApiPreUploadResponse,
    partseq: int,
    chunk_data: bytes,
) -> None:
    """上传单个分片"""
    logger.debug("Uploading part %d", partseq + 1)
    params = {
        "method": "upload",
        "type": "tmpfile",
        "path": pre_resp.path,
        "uploadid": pre_resp.uploadid,
        "partseq": str(partseq),
    }
    files = {"file": ("blob", chunk_data)}
    await transport.request("POST", url_prefix, params=params, files=files)


async def _api_pre_create(
    transport: _HttpTransport,
    path: str,
    size: int,
    block_list: list[str],
) -> _ApiPreUploadResponse:
    """预上传 - 通知云端新建上传任务"""
    params = {"method": "precreate"}
    data = {
        "path": path,
        "size": size,
        "isdir": "0",
        "autoinit": "1",
        "rtype": "3",  # 覆盖
        "block_list": json.dumps(block_list),
    }
    resp = await transport.request("POST", "https://pan.baidu.com/rest/2.0/xpan/file", params=params, data=data)
    return _ApiPreUploadResponse(
        path=path,
        uploadid=resp["uploadid"],
        block_list=resp.get("block_list", []),
        block_list_md5=block_list,
        file_size=size,
    )


async def _get_upload_server(transport: _HttpTransport, path: str, uploadid: str) -> str:
    """获取上传服务器域名"""
    params = {
        "method": "locateupload",
        "appid": "250528",
        "path": path,
        "uploadid": uploadid,
        "upload_version": "2.0",
    }
    resp = await transport.request("GET", "https://pan.baidu.com/rest/2.0/xpan/file", params=params)
    servers = resp.get("servers", [])
    if not servers:
        raise BaiduPanAPIError(-1, "No upload server available")

    for s in servers:
        server_url = s.get("server", "")
        if server_url.startswith("https"):
            return server_url

    # 降级：返回第一个有效 server
    for s in servers:
        server_url = s.get("server", "")
        if server_url:
            return server_url

    raise BaiduPanAPIError(-1, "No valid upload server in response")


async def _api_create_file(
    transport: _HttpTransport,
    path: str,
    uploadid: str,
    size: int,
    block_list: list[str],
) -> UploadResult:
    """合并分片完成上传"""
    params = {"method": "create"}
    data = {
        "path": path,
        "size": size,
        "isdir": "0",
        "rtype": "3",
        "uploadid": uploadid,
        "block_list": json.dumps(block_list),
    }
    resp = await transport.request("POST", "https://pan.baidu.com/rest/2.0/xpan/file", params=params, data=data)
    return UploadResult(
        fs_id=resp.get("fs_id"),
        md5=resp.get("md5"),
        path=resp.get("path"),
        size=resp.get("size"),
    )


async def _calculate_file_md5_blocks(file_path: Path) -> list[str]:
    """异步计算文件分片 MD5"""
    file_size = (await aiofiles.os.stat(file_path)).st_size

    async with aiofiles.open(file_path, "rb") as f:
        if file_size <= BLOCK_SIZE:
            content = await f.read()
            return [hashlib.md5(content).hexdigest()]

        md5_list: list[str] = []
        while True:
            chunk = await f.read(BLOCK_SIZE)
            if not chunk:
                break
            md5_list.append(hashlib.md5(chunk).hexdigest())

    return md5_list


def _calculate_bytes_md5_blocks(data: bytes) -> list[str]:
    """计算 bytes 对象的分片 MD5"""
    if len(data) <= BLOCK_SIZE:
        return [hashlib.md5(data).hexdigest()]

    md5_list: list[str] = []
    for i in range(0, len(data), BLOCK_SIZE):
        md5_list.append(hashlib.md5(data[i : i + BLOCK_SIZE]).hexdigest())
    return md5_list



"""上传模块异步测试"""

from __future__ import annotations

import hashlib
import json
from pathlib import Path, PurePosixPath

import httpx
import pytest
import respx

from abdds._http import _HttpTransport
from abdds.errors import BaiduPanAPIError
from abdds.models import UploadResult
from abdds.upload import (
    BLOCK_SIZE,
    _calculate_bytes_md5_blocks,
    _calculate_file_md5_blocks,
    _get_upload_server,
    upload,
)


@pytest.fixture
def transport() -> _HttpTransport:
    t = _HttpTransport(max_retries=0, timeout=(5, 10))
    t.set_access_token("test_token")
    return t


class TestCalculateFileMd5Blocks:
    async def test_small_file_single_block(self, tmp_path: Path):
        f = tmp_path / "small.bin"
        content = b"hello world"
        f.write_bytes(content)

        result = await _calculate_file_md5_blocks(f)
        assert len(result) == 1
        assert result[0] == hashlib.md5(content).hexdigest()

    async def test_large_file_multiple_blocks(self, tmp_path: Path):
        f = tmp_path / "large.bin"
        block1 = b"\x00" * BLOCK_SIZE
        block2 = b"\x01" * BLOCK_SIZE
        f.write_bytes(block1 + block2)

        result = await _calculate_file_md5_blocks(f)
        assert len(result) == 2
        assert result[0] == hashlib.md5(block1).hexdigest()
        assert result[1] == hashlib.md5(block2).hexdigest()

    async def test_exact_block_size(self, tmp_path: Path):
        f = tmp_path / "exact.bin"
        content = b"\xAA" * BLOCK_SIZE
        f.write_bytes(content)

        result = await _calculate_file_md5_blocks(f)
        assert len(result) == 1


class TestCalculateBytesMd5Blocks:
    def test_small_bytes(self):
        data = b"hello"
        result = _calculate_bytes_md5_blocks(data)
        assert len(result) == 1
        assert result[0] == hashlib.md5(data).hexdigest()

    def test_large_bytes(self):
        data = b"\x00" * (BLOCK_SIZE * 2 + 100)
        result = _calculate_bytes_md5_blocks(data)
        assert len(result) == 3
        assert result[0] == hashlib.md5(data[:BLOCK_SIZE]).hexdigest()
        assert result[1] == hashlib.md5(data[BLOCK_SIZE:BLOCK_SIZE*2]).hexdigest()
        assert result[2] == hashlib.md5(data[BLOCK_SIZE*2:]).hexdigest()


def _add_small_upload_mocks():
    """为小文件上传添加 mock 响应"""
    # precreate
    respx.post("https://pan.baidu.com/rest/2.0/xpan/file").mock(
        return_value=httpx.Response(200, json={"errno": 0, "uploadid": "uid123", "block_list": [0]})
    )
    # locate upload server
    respx.get("https://pan.baidu.com/rest/2.0/xpan/file").mock(
        return_value=httpx.Response(200, json={"errno": 0, "servers": [{"server": "https://upload.example.com"}]})
    )
    # direct upload
    respx.post("https://upload.example.com/rest/2.0/pcs/file").mock(
        return_value=httpx.Response(200, json={
            "errno": 0,
            "fs_id": 456,
            "md5": "md5val",
            "path": "/apps/test/small.txt",
            "size": 5,
        })
    )


class TestUploadWithPath:
    @respx.mock
    async def test_small_file(self, tmp_path: Path, transport: _HttpTransport):
        f = tmp_path / "small.txt"
        f.write_text("hello")

        _add_small_upload_mocks()

        result = await upload(transport, f, PurePosixPath("/small.txt"), "/apps/test")
        assert isinstance(result, UploadResult)
        assert result.fs_id == 456

    @respx.mock
    async def test_large_file(self, tmp_path: Path, transport: _HttpTransport):
        f = tmp_path / "large.bin"
        f.write_bytes(b"\x00" * (BLOCK_SIZE + 1))

        # precreate
        respx.post("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            return_value=httpx.Response(200, json={"errno": 0, "uploadid": "uid456", "block_list": [0, 1]})
        )
        # locate upload server
        respx.get("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            return_value=httpx.Response(200, json={"errno": 0, "servers": [{"server": "https://upload.example.com"}]})
        )
        # part uploads (2 parts)
        respx.post("https://upload.example.com/rest/2.0/pcs/superfile2").mock(
            return_value=httpx.Response(200, json={"errno": 0})
        )
        # merge - 注意: precreate 和 merge 都用 POST xpan/file，需要区分
        # respx 会按注册顺序匹配
        respx.post("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            side_effect=[
                httpx.Response(200, json={"errno": 0, "uploadid": "uid456", "block_list": [0, 1]}),
                httpx.Response(200, json={
                    "errno": 0,
                    "fs_id": 789,
                    "md5": "merged_md5",
                    "path": "/apps/test/large.bin",
                    "size": BLOCK_SIZE + 1,
                }),
            ]
        )

        result = await upload(transport, f, PurePosixPath("/large.bin"), "/apps/test")
        assert result.fs_id == 789

    async def test_file_not_found(self, transport: _HttpTransport):
        with pytest.raises(FileNotFoundError):
            await upload(transport, Path("/nonexistent/file.txt"), PurePosixPath("/file.txt"), "/apps/test")


class TestUploadWithBytes:
    @respx.mock
    async def test_small_bytes(self, transport: _HttpTransport):
        _add_small_upload_mocks()

        result = await upload(transport, b"hello", PurePosixPath("/small.txt"), "/apps/test")
        assert isinstance(result, UploadResult)
        assert result.fs_id == 456

    @respx.mock
    async def test_large_bytes(self, transport: _HttpTransport):
        data = b"\x00" * (BLOCK_SIZE + 1)

        # precreate + merge
        respx.post("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            side_effect=[
                httpx.Response(200, json={"errno": 0, "uploadid": "uid456", "block_list": [0, 1]}),
                httpx.Response(200, json={"errno": 0, "fs_id": 789, "md5": "m", "path": "/apps/test/f", "size": BLOCK_SIZE + 1}),
            ]
        )
        # locate upload server
        respx.get("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            return_value=httpx.Response(200, json={"errno": 0, "servers": [{"server": "https://upload.example.com"}]})
        )
        # part uploads
        respx.post("https://upload.example.com/rest/2.0/pcs/superfile2").mock(
            return_value=httpx.Response(200, json={"errno": 0})
        )

        result = await upload(transport, data, PurePosixPath("/f"), "/apps/test")
        assert result.fs_id == 789


class TestUploadWithGenerator:
    @respx.mock
    async def test_small_generator(self, transport: _HttpTransport):
        _add_small_upload_mocks()

        def gen():
            yield b"hello"

        result = await upload(transport, gen(), PurePosixPath("/small.txt"), "/apps/test")
        assert isinstance(result, UploadResult)

    @respx.mock
    async def test_small_async_generator(self, transport: _HttpTransport):
        _add_small_upload_mocks()

        async def gen():
            yield b"hello"

        result = await upload(transport, gen(), PurePosixPath("/small.txt"), "/apps/test")
        assert isinstance(result, UploadResult)

    @respx.mock
    async def test_multi_chunk_async_generator(self, transport: _HttpTransport):
        # 生成超过 BLOCK_SIZE 的数据
        respx.post("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            side_effect=[
                httpx.Response(200, json={"errno": 0, "uploadid": "uid", "block_list": [0, 1]}),
                httpx.Response(200, json={"errno": 0, "fs_id": 999, "md5": "m", "path": "/apps/test/f", "size": BLOCK_SIZE + 100}),
            ]
        )
        respx.get("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            return_value=httpx.Response(200, json={"errno": 0, "servers": [{"server": "https://upload.example.com"}]})
        )
        respx.post("https://upload.example.com/rest/2.0/pcs/superfile2").mock(
            return_value=httpx.Response(200, json={"errno": 0})
        )

        async def gen():
            yield b"\x00" * BLOCK_SIZE
            yield b"\x01" * 100

        result = await upload(transport, gen(), PurePosixPath("/f"), "/apps/test")
        assert result.fs_id == 999


class TestUploadInvalidType:
    async def test_invalid_file_from_type(self, transport: _HttpTransport):
        with pytest.raises(TypeError, match="file_from must be"):
            await upload(transport, "invalid_string", PurePosixPath("/f"), "/apps/test")  # type: ignore


class TestUploadEmptyData:
    async def test_empty_file_raises(self, transport: _HttpTransport, tmp_path: Path):
        """上传空文件 (0 bytes) 应抛 ValueError"""
        f = tmp_path / "empty.bin"
        f.write_bytes(b"")
        with pytest.raises(ValueError, match="Cannot upload empty file"):
            await upload(transport, f, PurePosixPath("/empty.bin"), "/apps/test")

    async def test_empty_bytes_raises(self, transport: _HttpTransport):
        """上传空 bytes 应抛 ValueError"""
        with pytest.raises(ValueError, match="Cannot upload empty data"):
            await upload(transport, b"", PurePosixPath("/empty.bin"), "/apps/test")

    async def test_empty_generator_raises(self, transport: _HttpTransport):
        """上传空生成器应抛 ValueError"""
        def empty_gen():
            return
            yield  # 使其成为生成器

        with pytest.raises(ValueError, match="Cannot upload empty generator"):
            await upload(transport, empty_gen(), PurePosixPath("/empty.bin"), "/apps/test")

    async def test_empty_async_generator_raises(self, transport: _HttpTransport):
        """上传空异步生成器应抛 ValueError"""
        async def empty_gen():
            return
            yield  # 使其成为异步生成器

        with pytest.raises(ValueError, match="Cannot upload empty async generator"):
            await upload(transport, empty_gen(), PurePosixPath("/empty.bin"), "/apps/test")


class TestGetUploadServer:
    @respx.mock
    async def test_prefer_https(self, transport: _HttpTransport):
        respx.get("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            return_value=httpx.Response(200, json={
                "errno": 0,
                "servers": [
                    {"server": "http://upload1.example.com"},
                    {"server": "https://upload2.example.com"},
                ],
            })
        )

        host = await _get_upload_server(transport, "/apps/test/f.txt", "uid")
        assert host == "https://upload2.example.com"

    @respx.mock
    async def test_no_https_available(self, transport: _HttpTransport):
        respx.get("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            return_value=httpx.Response(200, json={
                "errno": 0,
                "servers": [{"server": "http://upload1.example.com"}],
            })
        )

        host = await _get_upload_server(transport, "/apps/test/f.txt", "uid")
        assert host == "http://upload1.example.com"

    @respx.mock
    async def test_no_server_raises(self, transport: _HttpTransport):
        respx.get("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            return_value=httpx.Response(200, json={"errno": 0, "servers": []})
        )

        with pytest.raises(BaiduPanAPIError) as exc_info:
            await _get_upload_server(transport, "/apps/test/f.txt", "uid")
        assert "No upload server" in str(exc_info.value)

    @respx.mock
    async def test_server_missing_key_raises(self, transport: _HttpTransport):
        """servers 列表中的条目缺少 server 字段应报错"""
        respx.get("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            return_value=httpx.Response(200, json={"errno": 0, "servers": [{"port": 443}]})
        )

        with pytest.raises(BaiduPanAPIError) as exc_info:
            await _get_upload_server(transport, "/apps/test/f.txt", "uid")
        assert "No valid upload server" in str(exc_info.value)



"""异常体系测试"""

from abdds.errors import (
    BaiduPanAPIError,
    BaiduPanError,
    BaiduPanNetworkError,
    TokenExpiredError,
)


class TestExceptionHierarchy:
    """异常继承关系"""

    def test_base_is_exception(self):
        assert issubclass(BaiduPanError, Exception)

    def test_network_inherits_base(self):
        assert issubclass(BaiduPanNetworkError, BaiduPanError)

    def test_api_inherits_base(self):
        assert issubclass(BaiduPanAPIError, BaiduPanError)

    def test_token_expired_inherits_base(self):
        assert issubclass(TokenExpiredError, BaiduPanError)


class TestBaiduPanAPIError:
    def test_attributes(self):
        err = BaiduPanAPIError(errno=31023, errmsg="file does not exist", request_id="req_123")
        assert err.errno == 31023
        assert err.errmsg == "file does not exist"
        assert err.request_id == "req_123"

    def test_str_with_request_id(self):
        err = BaiduPanAPIError(errno=31023, errmsg="file does not exist", request_id="req_123")
        assert "[errno:31023]" in str(err)
        assert "file does not exist" in str(err)
        assert "req_123" in str(err)

    def test_str_without_request_id(self):
        err = BaiduPanAPIError(errno=31023, errmsg="file does not exist")
        assert "[errno:31023]" in str(err)
        assert "request_id" not in str(err)

    def test_default_errmsg(self):
        err = BaiduPanAPIError(errno=-1)
        assert str(err) == "[errno:-1] "


class TestBaiduPanNetworkError:
    def test_attributes(self):
        err = BaiduPanNetworkError("Connection failed", url="https://example.com", status_code=503)
        assert err.url == "https://example.com"
        assert err.status_code == 503
        assert "Connection failed" in str(err)

    def test_defaults(self):
        err = BaiduPanNetworkError("timeout")
        assert err.url == ""
        assert err.status_code is None


class TestTokenExpiredError:
    def test_default_message(self):
        err = TokenExpiredError()
        assert "expired" in str(err).lower() or "invalid" in str(err).lower()

    def test_custom_message(self):
        err = TokenExpiredError("custom msg")
        assert str(err) == "custom msg"



"""下载相关逻辑 - 异步流式下载、断点续传"""

from __future__ import annotations

import asyncio
import logging
from pathlib import Path
from typing import TYPE_CHECKING, AsyncGenerator

import aiofiles
import httpx

from ._http import UserAgent
from .errors import BaiduPanNetworkError

if TYPE_CHECKING:
    from ._http import _HttpTransport

logger = logging.getLogger("abdds")


async def download(
    transport: _HttpTransport,
    dlink: str,
    file_to: Path | None = None,
    chunk_size: int = 1024 * 1024,
    max_retries: int = 5,
) -> AsyncGenerator[bytes, None] | Path:
    """
    异步下载文件

    - file_to=None: 返回异步字节迭代器 (流式下载)
    - file_to=Path: 下载到本地文件，返回 Path

    Args:
        transport: 异步 HTTP 传输层
        dlink: 下载链接
        file_to: 本地保存路径 (Path)，为 None 时返回迭代器
        chunk_size: 分块大小，默认 1MB
        max_retries: 最大重试次数，默认 5

    Returns:
        AsyncGenerator[bytes] 当 file_to=None，Path 当 file_to 指定路径

    Raises:
        TypeError: file_to 类型不正确
    """
    if file_to is not None and not isinstance(file_to, Path):
        raise TypeError(
            f"file_to must be Path or None, got {type(file_to).__name__}"
        )
    if chunk_size <= 0:
        raise ValueError(f"chunk_size must be positive, got {chunk_size}")

    stream = _download_stream(transport, dlink, chunk_size, max_retries)

    if file_to is None:
        return stream

    file_to.parent.mkdir(parents=True, exist_ok=True)

    async with aiofiles.open(file_to, "wb") as f:
        async for chunk in stream:
            await f.write(chunk)

    logger.info("Downloaded to %s", file_to)
    return file_to


async def _download_stream(
    transport: _HttpTransport,
    dlink: str,
    chunk_size: int = 1024 * 1024,
    max_retries: int = 5,
) -> AsyncGenerator[bytes, None]:
    """
    通过 dlink 异步流式下载文件，支持断点续传和指数重试
    """
    downloaded_bytes = 0
    retry_count = 0

    while True:
        try:
            headers = {"User-Agent": UserAgent}

            if downloaded_bytes > 0:
                headers["Range"] = f"bytes={downloaded_bytes}-"
                logger.info("Resuming download from byte %d...", downloaded_bytes)

            async with transport.client.stream(
                "GET",
                dlink,
                headers=headers,
                params={"access_token": transport._access_token},
                timeout=httpx.Timeout(60, connect=15),
            ) as response:
                response.raise_for_status()

                # 请求了 Range 但服务器返回 200，说明不支持断点续传
                if downloaded_bytes > 0 and response.status_code == 200:
                    logger.warning("Server does not support Range, restarting from beginning")
                    downloaded_bytes = 0

                async for chunk in response.aiter_bytes(chunk_size=chunk_size):
                    if chunk:
                        yield chunk
                        downloaded_bytes += len(chunk)

            break  # 下载完成

        except httpx.RequestError as e:
            retry_count += 1
            if retry_count > max_retries:
                raise BaiduPanNetworkError(
                    f"Download failed after {max_retries} retries "
                    f"({downloaded_bytes} bytes downloaded): {e}",
                    url=dlink,
                ) from e

            wait_time = 2 ** (retry_count - 1)
            logger.warning(
                "Download interrupted at %d bytes: %s. Retrying in %ds...",
                downloaded_bytes, e, wait_time,
            )
            await asyncio.sleep(wait_time)



"""下载模块异步测试"""

from __future__ import annotations

from pathlib import Path
from unittest.mock import AsyncMock, patch

import httpx
import pytest
import respx

from abdds._http import _HttpTransport
from abdds.download import download
from abdds.errors import BaiduPanNetworkError


DLINK = "https://dlink.example.com/file"


@pytest.fixture
def transport() -> _HttpTransport:
    t = _HttpTransport(max_retries=0, timeout=(5, 10))
    t.set_access_token("test_token")
    return t


class TestDownloadStream:
    @respx.mock
    async def test_file_to_none_returns_async_generator(self, transport: _HttpTransport):
        content = b"hello world data"
        respx.get(DLINK).mock(
            return_value=httpx.Response(200, content=content)
        )

        result = await download(transport, DLINK, file_to=None)
        chunks = []
        async for chunk in result:
            chunks.append(chunk)
        assert b"".join(chunks) == content

    @respx.mock
    async def test_empty_file(self, transport: _HttpTransport):
        respx.get(DLINK).mock(
            return_value=httpx.Response(200, content=b"")
        )

        result = await download(transport, DLINK, file_to=None)
        chunks = []
        async for chunk in result:
            chunks.append(chunk)
        assert b"".join(chunks) == b""

    @respx.mock
    async def test_retry_on_failure(self, transport: _HttpTransport):
        content = b"retry data"
        route = respx.get(DLINK)
        route.side_effect = [
            httpx.ConnectError("lost"),
            httpx.Response(200, content=content),
        ]

        with patch("abdds.download.asyncio.sleep", new_callable=AsyncMock):
            result = await download(transport, DLINK, file_to=None, max_retries=3)
            chunks = []
            async for chunk in result:
                chunks.append(chunk)

        assert b"".join(chunks) == content

    @respx.mock
    async def test_max_retries_exceeded(self, transport: _HttpTransport):
        respx.get(DLINK).mock(
            side_effect=httpx.ConnectError("lost")
        )

        with patch("abdds.download.asyncio.sleep", new_callable=AsyncMock):
            with pytest.raises(BaiduPanNetworkError) as exc_info:
                result = await download(transport, DLINK, file_to=None, max_retries=2)
                chunks = []
                async for chunk in result:
                    chunks.append(chunk)

        assert "Download failed" in str(exc_info.value)


class TestDownloadFile:
    @respx.mock
    async def test_download_to_path(self, transport: _HttpTransport, tmp_path: Path):
        content = b"file content here"
        respx.get(DLINK).mock(
            return_value=httpx.Response(200, content=content)
        )

        local_path = tmp_path / "downloaded.bin"
        result = await download(transport, DLINK, file_to=local_path)

        assert result == local_path
        assert local_path.read_bytes() == content

    @respx.mock
    async def test_creates_parent_dirs(self, transport: _HttpTransport, tmp_path: Path):
        content = b"nested file"
        respx.get(DLINK).mock(
            return_value=httpx.Response(200, content=content)
        )

        local_path = tmp_path / "sub" / "dir" / "file.bin"
        result = await download(transport, DLINK, file_to=local_path)
        assert local_path.read_bytes() == content

    async def test_file_to_str_raises_typeerror(self, transport: _HttpTransport):
        """file_to 只接受 Path 或 None，str 应报 TypeError"""
        with pytest.raises(TypeError, match="file_to must be Path or None"):
            await download(transport, DLINK, file_to="/tmp/out.bin")  # type: ignore

    async def test_invalid_chunk_size_raises(self, transport: _HttpTransport):
        """chunk_size <= 0 应报 ValueError"""
        with pytest.raises(ValueError, match="chunk_size must be positive"):
            await download(transport, DLINK, file_to=None, chunk_size=0)

        with pytest.raises(ValueError, match="chunk_size must be positive"):
            await download(transport, DLINK, file_to=None, chunk_size=-1)



"""AsyncBaiduPanClient 测试"""

from __future__ import annotations

import json
from pathlib import Path
from unittest.mock import AsyncMock

import httpx
import pytest
import respx

from abdds import AsyncBaiduPanClient, BaiduPanNetworkError, TokenExpiredError
from abdds.models import ApiFileListItem, ApiFileMeta, ApiQuotaInfo


class TestClientInit:
    def test_basic_init(self, tmp_config_dir: Path):
        client = AsyncBaiduPanClient("id", "secret", "myapp", config_dir=tmp_config_dir)
        assert client.pan_dir_base == "/apps/myapp"
        assert client.access_token is None

    async def test_context_manager(self, tmp_config_dir: Path):
        async with AsyncBaiduPanClient("id", "secret", "myapp", config_dir=tmp_config_dir) as client:
            assert client is not None
        # 关闭后 client 应该被关闭
        assert client._transport._client is None

    def test_auth_url(self, tmp_config_dir: Path):
        client = AsyncBaiduPanClient("my_client_id", "secret", "app", config_dir=tmp_config_dir)
        url = client.auth_url
        assert "my_client_id" in url
        assert "authorize" in url


class TestTokenManagement:
    @respx.mock
    async def test_fetch_token(self, tmp_config_dir: Path):
        respx.get("https://openapi.baidu.com/oauth/2.0/token").mock(
            return_value=httpx.Response(200, json={
                "access_token": "new_access",
                "refresh_token": "new_refresh",
                "expires_in": 2592000,
            })
        )

        client = AsyncBaiduPanClient("id", "secret", "app", config_dir=tmp_config_dir)
        await client.fetch_token("auth_code_123")

        assert client.access_token == "new_access"
        assert client._refresh_token == "new_refresh"

        # token 文件应已创建
        token_file = tmp_config_dir / "access_token_id.json"
        assert token_file.exists()
        saved = json.loads(token_file.read_text())
        assert saved["access_token"] == "new_access"

    @respx.mock
    async def test_refresh_token(self, tmp_config_dir: Path):
        respx.get("https://openapi.baidu.com/oauth/2.0/token").mock(
            return_value=httpx.Response(200, json={
                "access_token": "refreshed_access",
                "refresh_token": "refreshed_refresh",
                "expires_in": 2592000,
            })
        )

        client = AsyncBaiduPanClient("id", "secret", "app", config_dir=tmp_config_dir)
        client._refresh_token = "old_refresh"

        result = await client.refresh_token()
        assert result == "refreshed_access"
        assert client.access_token == "refreshed_access"

    async def test_refresh_token_without_refresh_token(self, tmp_config_dir: Path):
        client = AsyncBaiduPanClient("id", "secret", "app", config_dir=tmp_config_dir)
        with pytest.raises(TokenExpiredError):
            await client.refresh_token()

    @respx.mock
    async def test_fetch_token_network_error(self, tmp_config_dir: Path):
        respx.get("https://openapi.baidu.com/oauth/2.0/token").mock(
            side_effect=httpx.ConnectError("failed")
        )

        client = AsyncBaiduPanClient("id", "secret", "app", config_dir=tmp_config_dir)
        with pytest.raises(BaiduPanNetworkError):
            await client.fetch_token("code")

    async def test_load_token(self, tmp_config_dir: Path):
        token_file = tmp_config_dir / "access_token_id.json"
        token_file.write_text(json.dumps({
            "access_token": "loaded_access",
            "refresh_token": "loaded_refresh",
        }))

        client = AsyncBaiduPanClient("id", "secret", "app", config_dir=tmp_config_dir)
        await client._load_token()
        assert client.access_token == "loaded_access"


class TestBusinessAPI:
    @respx.mock
    async def test_get_quota(self, mock_client: AsyncBaiduPanClient):
        respx.get("https://pan.baidu.com/api/quota").mock(
            return_value=httpx.Response(200, json={"errno": 0, "total": 214748364800, "used": 1073741824})
        )

        quota = await mock_client.get_quota()
        assert isinstance(quota, ApiQuotaInfo)
        assert quota.total == 214748364800
        assert quota.used == 1073741824

    @respx.mock
    async def test_get_file_metas(self, mock_client: AsyncBaiduPanClient):
        respx.get("https://pan.baidu.com/rest/2.0/xpan/multimedia").mock(
            return_value=httpx.Response(200, json={
                "errno": 0,
                "list": [
                    {
                        "fs_id": 123,
                        "filename": "test.txt",
                        "path": "/apps/test/test.txt",
                        "size": 1024,
                        "md5": "abc123",
                        "dlink": "https://dlink.example.com/f",
                    }
                ],
            })
        )

        metas = await mock_client.get_file_metas([123])
        assert len(metas) == 1
        assert isinstance(metas[0], ApiFileMeta)
        assert metas[0].fs_id == 123
        assert metas[0].dlink == "https://dlink.example.com/f"

    async def test_get_file_metas_empty_fsids(self, mock_client: AsyncBaiduPanClient):
        result = await mock_client.get_file_metas([])
        assert result == []

    @respx.mock
    async def test_get_file_list(self, mock_client: AsyncBaiduPanClient):
        respx.get("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            return_value=httpx.Response(200, json={
                "errno": 0,
                "list": [
                    {
                        "fs_id": 1,
                        "server_filename": "dir1",
                        "path": "/apps/test/dir1",
                        "size": 0,
                        "md5": "",
                        "isdir": 1,
                    },
                    {
                        "fs_id": 2,
                        "server_filename": "file.txt",
                        "path": "/apps/test/file.txt",
                        "size": 50,
                        "md5": "md5val",
                        "isdir": 0,
                    },
                ],
            })
        )

        items = await mock_client.get_file_list("/apps/test/")
        assert len(items) == 2
        assert isinstance(items[0], ApiFileListItem)
        assert items[0].is_dir == 1
        assert items[1].filename == "file.txt"

    @respx.mock
    async def test_delete_files(self, mock_client: AsyncBaiduPanClient):
        route = respx.post("https://pan.baidu.com/rest/2.0/xpan/file").mock(
            return_value=httpx.Response(200, json={"errno": 0})
        )

        await mock_client.delete_files(["/apps/test/f.txt"])
        assert route.called

    async def test_delete_files_empty_list(self, mock_client: AsyncBaiduPanClient):
        # 空列表不应发起请求
        await mock_client.delete_files([])

    @respx.mock
    async def test_no_token_raises(self, tmp_config_dir: Path):
        respx.get("https://pan.baidu.com/api/quota").mock(
            return_value=httpx.Response(200, json={"errno": 0, "total": 100, "used": 0})
        )

        client = AsyncBaiduPanClient("id", "secret", "app", config_dir=tmp_config_dir)
        with pytest.raises(TokenExpiredError):
            await client.get_quota()


class TestTokenEdgeCases:
    @respx.mock
    async def test_fetch_token_api_error(self, tmp_config_dir: Path):
        """API 返回 error 字段时应抛 TokenExpiredError"""
        respx.get("https://openapi.baidu.com/oauth/2.0/token").mock(
            return_value=httpx.Response(200, json={"error": "invalid_code", "error_description": "Authorization code is invalid"})
        )

        client = AsyncBaiduPanClient("id", "secret", "app", config_dir=tmp_config_dir)
        with pytest.raises(TokenExpiredError, match="Authorization code is invalid"):
            await client.fetch_token("bad_code")

    @respx.mock
    async def test_refresh_token_api_error(self, tmp_config_dir: Path):
        """刷新 token 时 API 返回错误"""
        respx.get("https://openapi.baidu.com/oauth/2.0/token").mock(
            return_value=httpx.Response(200, json={"error": "invalid_refresh_token", "error_description": "Refresh token has expired"})
        )

        client = AsyncBaiduPanClient("id", "secret", "app", config_dir=tmp_config_dir)
        client._refresh_token = "old_refresh"
        with pytest.raises(TokenExpiredError, match="Refresh token has expired"):
            await client.refresh_token()

    @respx.mock
    async def test_save_token_missing_access_token(self, tmp_config_dir: Path):
        """_save_token 收到不含 access_token 的数据应报错"""
        client = AsyncBaiduPanClient("id", "secret", "app", config_dir=tmp_config_dir)
        with pytest.raises(TokenExpiredError, match="missing access_token"):
            await client._save_token({"refresh_token": "rt"})

    async def test_load_token_corrupted_file(self, tmp_config_dir: Path):
        """token 文件损坏时应优雅降级"""
        token_file = tmp_config_dir / "access_token_id.json"
        token_file.write_text("NOT VALID JSON{{{")

        client = AsyncBaiduPanClient("id", "secret", "app", config_dir=tmp_config_dir)
        await client._load_token()
        assert client.access_token is None


class TestUploadDownloadEdgeCases:
    async def test_upload_file_from_type_error(self, mock_client: AsyncBaiduPanClient):
        """file_from 传入 str 应抛 TypeError"""
        with pytest.raises(TypeError, match="file_from must be"):
            await mock_client.upload("not_a_path")  # type: ignore

    async def test_upload_file_to_type_error(self, mock_client: AsyncBaiduPanClient):
        """file_to 传入 str 应抛 TypeError"""
        with pytest.raises(TypeError, match="file_to must be"):
            await mock_client.upload(Path("f.txt"), file_to="/str_path")  # type: ignore

    async def test_download_file_to_type_error(self, mock_client: AsyncBaiduPanClient):
        """download file_to 传入 str 应抛 TypeError"""
        with pytest.raises(TypeError, match="file_to must be"):
            await mock_client.download("https://dlink", file_to="/str_path")  # type: ignore



"""HTTP 传输层 - 异步 Session 管理、请求封装、重试策略"""

from __future__ import annotations

import json
import logging
from typing import Any, Awaitable, Callable

import httpx

from .errors import BaiduPanAPIError, BaiduPanNetworkError, TokenExpiredError

logger = logging.getLogger("abdds")

UserAgent = "netdisk;2.2.51.6;netdisk;10.0;PC;PC;Mac OS X 10.15.7;en-US"


def _truncate_data(data: Any, max_len: int = 500) -> Any:
    """递归截断超长字符串和二进制数据，用于日志安全输出"""
    if isinstance(data, dict):
        return {k: _truncate_data(v, max_len) for k, v in data.items()}
    if isinstance(data, (list, tuple, set)):
        return [_truncate_data(i, max_len) for i in data]
    if isinstance(data, (bytes, bytearray)):
        return f"<Binary Data: {len(data)} bytes>"
    if isinstance(data, str) and len(data) > max_len:
        return f"{data[:max_len]}... [truncated]"
    return data


class _HttpTransport:
    """封装 httpx.AsyncClient，提供自动 Token 注入、重试和错误处理"""

    def __init__(
        self,
        max_retries: int = 3,
        timeout: tuple[int, int] = (100, 600),
    ) -> None:
        self._max_retries = max_retries
        self._timeout = timeout
        self._client: httpx.AsyncClient | None = None
        self._closed: bool = False
        self._access_token: str | None = None
        self._on_token_expired: Callable[[], Awaitable[str | None]] | None = None

    def _init_client(self) -> httpx.AsyncClient:
        transport = httpx.AsyncHTTPTransport(
            retries=self._max_retries,
        )
        client = httpx.AsyncClient(
            transport=transport,
            timeout=httpx.Timeout(self._timeout[1], connect=self._timeout[0]),
            headers={"User-Agent": UserAgent},
            follow_redirects=True,
        )
        return client

    @property
    def client(self) -> httpx.AsyncClient:
        """获取或初始化 AsyncClient"""
        if self._closed:
            raise RuntimeError("Transport is closed, cannot make requests")
        if self._client is None:
            self._client = self._init_client()
        return self._client

    def set_access_token(self, token: str | None) -> None:
        self._access_token = token

    def set_token_expired_callback(
        self, callback: Callable[[], Awaitable[str | None]]
    ) -> None:
        """设置 Token 过期时的异步刷新回调，回调应返回新的 access_token"""
        self._on_token_expired = callback

    async def request(
        self, method: str, url: str, **kwargs: Any
    ) -> dict | httpx.Response:
        """
        核心异步请求方法：
        1. 自动注入 access_token
        2. 处理网络层错误 (由 httpx transport retries 处理重试)
        3. 处理业务层错误 (解析 errno)
        4. Token 失效时自动刷新并重试一次
        """
        logger.debug("method:%s url:%s kwargs:%s", method, url, _truncate_data(kwargs))

        if self._closed:
            raise RuntimeError("Transport is closed, cannot make requests")

        if not self._access_token:
            raise TokenExpiredError("Access token not initialized, please login first")

        params = kwargs.pop("params", {})
        params["access_token"] = self._access_token

        try:
            response = await self.client.request(method, url, params=params, **kwargs)
            response.raise_for_status()

            # 流式响应直接返回
            if kwargs.get("stream"):
                return response  # type: ignore[return-value]

            try:
                data = response.json()
            except (json.JSONDecodeError, ValueError):
                raise BaiduPanAPIError(-1, "Invalid JSON response")
            logger.debug("method:%s url:%s response:%s", method, url, _truncate_data(data))

        except httpx.HTTPStatusError as e:
            raise BaiduPanNetworkError(
                f"HTTP {e.response.status_code}: {e}", url=url,
                status_code=e.response.status_code,
            ) from e
        except httpx.RequestError as e:
            raise BaiduPanNetworkError(
                f"Request failed: {e}", url=url
            ) from e

        # 业务错误检查
        errno = data.get("errno", 0)
        if errno != 0:
            # Token 失效 (errno: -6 或 11)
            if errno in (-6, 11) and self._on_token_expired:
                logger.warning("Token expired during request, refreshing...")
                new_token = await self._on_token_expired()
                if new_token:
                    params["access_token"] = new_token
                    try:
                        response = await self.client.request(
                            method, url, params=params, **kwargs
                        )
                        response.raise_for_status()
                        data = response.json()
                    except httpx.RequestError as e:
                        raise BaiduPanNetworkError(
                            f"Retry after token refresh failed: {e}", url=url
                        ) from e
                    except (json.JSONDecodeError, ValueError):
                        raise BaiduPanAPIError(-1, "Invalid JSON response")

                    if data.get("errno", 0) == 0:
                        return data

                # 刷新后仍失败
                raise BaiduPanAPIError(
                    data.get("errno", errno),
                    data.get("errmsg", "Token refresh failed"),
                    request_id=data.get("request_id", ""),
                )

            raise BaiduPanAPIError(
                errno,
                data.get("errmsg", "Unknown error"),
                request_id=data.get("request_id", ""),
            )

        return data

    async def close(self) -> None:
        self._closed = True
        if self._client is not None:
            await self._client.aclose()
            self._client = None



"""AsyncBaiduPanClient - 百度网盘异步 Python SDK 主类"""

from __future__ import annotations

import json
import logging
import stat
import time
from pathlib import Path, PurePosixPath
from typing import AsyncGenerator, Awaitable, Callable
from urllib.parse import quote

import aiofiles
import httpx

from ._http import _HttpTransport
from .download import download as _download
from .errors import BaiduPanNetworkError, TokenExpiredError
from .models import ApiFileListItem, ApiFileMeta, ApiQuotaInfo, UploadResult
from .upload import upload as _upload
from .upload import UploadSource, _is_async_generator, _is_generator

logger = logging.getLogger("abdds")


class AsyncBaiduPanClient:
    """
    百度网盘异步 Python SDK 客户端

    具备自动重试、Token 管理、流式下载和分片上传。

    Usage::

        async with AsyncBaiduPanClient(client_id, client_secret, app_name) as client:
            if not client.access_token:
                print(client.auth_url)
                code = input("Enter code: ")
                await client.fetch_token(code)

            quota = await client.get_quota()
            result = await client.upload(Path("local_file.txt"))
    """

    HOST_OAUTH = "https://openapi.baidu.com/oauth/2.0"
    HOST_PAN = "https://pan.baidu.com"

    URL_QUOTA = f"{HOST_PAN}/api/quota"
    URL_FILE = f"{HOST_PAN}/rest/2.0/xpan/file"
    URL_MULTIMEDIA = f"{HOST_PAN}/rest/2.0/xpan/multimedia"

    def __init__(
        self,
        client_id: str,
        client_secret: str,
        app_name: str,
        config_dir: Path | None = None,
        max_retries: int = 3,
        timeout: tuple[int, int] | None = None,
    ) -> None:
        """
        初始化百度网盘异步客户端

        Args:
            client_id: 百度开放平台应用的 API Key
            client_secret: 百度开放平台应用的 Secret Key
            app_name: 应用名称
            config_dir: Token 存储目录，默认 ~/.baidupan
            max_retries: 请求重试次数，默认 3
            timeout: 请求超时 (connect, read)，默认 (100, 600)
        """
        self._client_id = client_id
        self._client_secret = client_secret
        self.app_name = app_name
        self.pan_dir_base = f"/apps/{app_name}"

        if config_dir is None:
            config_dir = Path.home() / ".baidupan"
        self.config_dir = config_dir
        self._token_file = self.config_dir / f"access_token_{client_id}.json"

        self._token_data: dict = {}
        self._access_token: str | None = None
        self._refresh_token: str | None = None

        # 初始化异步 HTTP 传输层
        self._transport = _HttpTransport(
            max_retries=max_retries,
            timeout=timeout or (100, 600),
        )
        self._transport.set_token_expired_callback(self._handle_token_expired)

        # 注意: _load_token 是异步方法，不能在 __init__ 中直接调用
        # 需要用户在初始化后手动调用 await client._load_token()
        # 或者使用 create 类方法

    @classmethod
    async def create(
        cls,
        client_id: str,
        client_secret: str,
        app_name: str,
        config_dir: Path | None = None,
        max_retries: int = 3,
        timeout: tuple[int, int] | None = None,
    ) -> AsyncBaiduPanClient:
        """异步工厂方法，初始化客户端并自动加载已保存的 Token"""
        instance = cls(client_id, client_secret, app_name, config_dir, max_retries, timeout)
        await instance._load_token()
        return instance

    # ---- 上下文管理器 ----

    async def close(self) -> None:
        """关闭客户端，释放资源"""
        await self._transport.close()

    async def __aenter__(self) -> AsyncBaiduPanClient:
        await self._load_token()
        return self

    async def __aexit__(
        self, exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: object,
    ) -> bool | None:
        await self.close()
        return False

    # ---- Token 管理 ----

    @property
    def access_token(self) -> str | None:
        """当前 access_token（只读）"""
        return self._access_token

    @property
    def auth_url(self) -> str:
        """生成用户授权 URL"""
        return (
            f"{self.HOST_OAUTH}/authorize?"
            f"response_type=code&client_id={self._client_id}&redirect_uri=oob&scope=basic,netdisk"
        )

    async def fetch_token(self, code: str) -> None:
        """
        通过授权码获取 Access Token

        Args:
            code: 用户授权后获得的授权码
        """
        url = f"{self.HOST_OAUTH}/token"
        params = {
            "grant_type": "authorization_code",
            "code": code,
            "client_id": self._client_id,
            "client_secret": self._client_secret,
            "redirect_uri": "oob",
        }
        try:
            resp = await self._transport.client.get(url, params=params, timeout=10)
            resp.raise_for_status()
            result = resp.json()
        except httpx.RequestError as e:
            raise BaiduPanNetworkError(f"Fetch token failed: {e}") from e

        # 检查 API 层面错误
        if "error" in result:
            raise TokenExpiredError(
                f"Fetch token failed: {result.get('error_description', result['error'])}"
            )
        await self._save_token(result)

    async def refresh_token(self) -> str:
        """
        刷新 Access Token

        Returns:
            新的 access_token

        Raises:
            TokenExpiredError: 无 refresh_token 可用
        """
        if not self._refresh_token:
            raise TokenExpiredError("No refresh token available, please re-login")

        url = f"{self.HOST_OAUTH}/token"
        params = {
            "grant_type": "refresh_token",
            "refresh_token": self._refresh_token,
            "client_id": self._client_id,
            "client_secret": self._client_secret,
        }
        try:
            resp = await self._transport.client.get(url, params=params, timeout=10)
            resp.raise_for_status()
            result = resp.json()
        except httpx.RequestError as e:
            raise BaiduPanNetworkError(f"Refresh token failed: {e}") from e

        # 检查 API 层面错误
        if "error" in result:
            raise TokenExpiredError(
                f"Refresh token failed: {result.get('error_description', result['error'])}"
            )
        await self._save_token(result)
        return self._access_token  # type: ignore[return-value]

    async def _handle_token_expired(self) -> str | None:
        """Token 过期回调，供 _HttpTransport 调用"""
        try:
            return await self.refresh_token()
        except (TokenExpiredError, BaiduPanNetworkError):
            logger.error("Token refresh failed in auto-retry")
            return None

    async def _load_token(self) -> None:
        """从本地文件异步加载 Token"""
        if not self._token_file.exists():
            return
        try:
            async with aiofiles.open(self._token_file, encoding="utf-8") as f:
                content = await f.read()
            data = json.loads(content)
            self._token_data = data
            self._access_token = data.get("access_token")
            self._refresh_token = data.get("refresh_token")
            self._transport.set_access_token(self._access_token)
        except (json.JSONDecodeError, IOError) as e:
            logger.error("Failed to load token: %s", e)

    async def _save_token(self, data: dict) -> None:
        """异步持久化保存 Token"""
        if "access_token" not in data:
            error_desc = data.get("error_description", data.get("Error", "Unknown error"))
            raise TokenExpiredError(f"Token response missing access_token: {error_desc}")

        self.config_dir.mkdir(parents=True, exist_ok=True)
        data["update_at"] = int(time.time())
        self._token_data = data
        self._access_token = data["access_token"]
        self._refresh_token = data.get("refresh_token")
        self._transport.set_access_token(self._access_token)

        async with aiofiles.open(self._token_file, "w", encoding="utf-8") as f:
            await f.write(json.dumps(data, indent=2))
        # 限制文件权限为仅拥有者可读写
        try:
            self._token_file.chmod(stat.S_IRUSR | stat.S_IWUSR)
        except OSError:
            pass  # Windows 不完全支持 chmod

        logger.info("Access token saved")

    # ---- 业务 API ----

    async def get_quota(self) -> ApiQuotaInfo:
        """
        获取网盘空间配额

        https://pan.baidu.com/union/doc/Cksg0s9ic
        """
        data = await self._transport.request("GET", self.URL_QUOTA)
        return ApiQuotaInfo(total=data.get("total", 0), used=data.get("used", 0))

    async def get_file_metas(self, fsids: list[int]) -> list[ApiFileMeta]:
        """
        获取文件元信息

        https://pan.baidu.com/union/doc/Fksg0sbcm

        Args:
            fsids: 文件 ID 列表
        """
        if not fsids:
            return []

        params = {"method": "filemetas", "fsids": json.dumps(fsids), "dlink": 1}
        data = await self._transport.request("GET", self.URL_MULTIMEDIA, params=params)
        items = data.get("list", [])

        return [
            ApiFileMeta(
                fs_id=item["fs_id"],
                filename=item["filename"],
                path=item["path"],
                size=item["size"],
                md5=item.get("md5", ""),
                dlink=item.get("dlink", ""),
            )
            for item in items
        ]

    async def get_file_list(self, path: str) -> list[ApiFileListItem]:
        """
        获取指定目录下的文件列表

        https://pan.baidu.com/union/doc/nksg0sat9

        Args:
            path: 目录路径
        """
        params = {"method": "list", "dir": quote(path)}
        data = await self._transport.request("GET", self.URL_FILE, params=params)
        items = data.get("list", [])

        return [
            ApiFileListItem(
                fs_id=item["fs_id"],
                filename=item["server_filename"],
                path=item["path"],
                size=item["size"],
                md5=item.get("md5", ""),
                is_dir=item.get("isdir", 0),
            )
            for item in items
        ]

    async def delete_files(self, file_paths: list[str]) -> None:
        """
        批量删除文件

        https://pan.baidu.com/union/doc/mksg0s9l4

        Args:
            file_paths: 要删除的文件路径列表
        """
        if not file_paths:
            return
        params = {"method": "filemanager", "opera": "delete"}
        data = {"async": 2, "filelist": json.dumps(file_paths)}
        await self._transport.request("POST", self.URL_FILE, params=params, data=data)
        logger.info("Deleted %d file(s)", len(file_paths))

    # ---- 上传 ----

    async def upload(
        self,
        file_from: UploadSource,
        file_to: Path | None = None,
    ) -> UploadResult:
        """
        异步上传数据到百度网盘

        - 小文件 (<=4MB): 单步上传
        - 大文件 (>4MB): 分片上传

        Args:
            file_from: 数据来源，支持 Path / bytes / Generator[bytes] / AsyncGenerator[bytes]
            file_to: 远程路径 (Path)，拼接在 /apps/{app_name} 之后。
                默认使用 file_from 的文件名 (Path 时) 或 "upload"

        Returns:
            UploadResult: 上传结果

        Raises:
            TypeError: file_from 或 file_to 类型不正确
        """
        # 类型校验
        if (
            not isinstance(file_from, (Path, bytes))
            and not _is_generator(file_from)
            and not _is_async_generator(file_from)
        ):
            raise TypeError(
                f"file_from must be Path, bytes, Generator[bytes], or AsyncGenerator[bytes], "
                f"got {type(file_from).__name__}"
            )
        if file_to is not None and not isinstance(file_to, Path):
            raise TypeError(
                f"file_to must be Path or None, got {type(file_to).__name__}"
            )

        if isinstance(file_from, Path):
            default_name = file_from.name
        else:
            default_name = "upload"

        if file_to is None:
            remote_path = PurePosixPath(f"/{default_name}")
        else:
            # 将 Path 转为 PurePosixPath 以确保 POSIX 风格
            remote_path = PurePosixPath(file_to.as_posix())
            if not remote_path.is_absolute():
                remote_path = PurePosixPath(f"/{remote_path}")

        return await _upload(self._transport, file_from, remote_path, self.pan_dir_base)

    # ---- 下载 ----

    async def download(
        self,
        dlink: str,
        file_to: Path | None = None,
        chunk_size: int = 1024 * 1024,
    ) -> AsyncGenerator[bytes, None] | Path:
        """
        异步下载文件

        - file_to=None: 返回异步字节迭代器 (流式下载)
        - file_to=Path: 下载到本地文件，返回 Path

        https://pan.baidu.com/union/doc/pkuo3snyp

        Args:
            dlink: 下载链接 (通过 get_file_metas 获取)
            file_to: 本地保存路径 (Path)，为 None 时返回迭代器
            chunk_size: 分块大小，默认 1MB

        Returns:
            AsyncGenerator[bytes] 当 file_to=None，Path 当 file_to 指定路径

        Raises:
            TypeError: file_to 类型不正确
        """
        if file_to is not None and not isinstance(file_to, Path):
            raise TypeError(
                f"file_to must be Path or None, got {type(file_to).__name__}"
            )
        return await _download(self._transport, dlink, file_to, chunk_size)



"""abdds 异常体系"""


class BaiduPanError(Exception):
    """SDK 基础异常"""


class BaiduPanNetworkError(BaiduPanError):
    """网络通信异常"""

    def __init__(
        self, message: str, *, url: str = "", status_code: int | None = None
    ):
        self.url = url
        self.status_code = status_code
        super().__init__(message)


class BaiduPanAPIError(BaiduPanError):
    """API 业务异常 (errno != 0)"""

    def __init__(
        self, errno: int, errmsg: str = "", *, request_id: str = ""
    ):
        self.errno = errno
        self.errmsg = errmsg
        self.request_id = request_id
        detail = f"[errno:{errno}] {errmsg}"
        if request_id:
            detail += f" (request_id:{request_id})"
        super().__init__(detail)


class TokenExpiredError(BaiduPanError):
    """Token 过期或无效"""

    def __init__(self, message: str = "Access token expired or invalid"):
        super().__init__(message)



"""abdds - 百度网盘异步 Python SDK (async-baidu-disk-sdk)

简单、有效、报错清晰的百度网盘异步 SDK。

Usage::

    from abdds import AsyncBaiduPanClient

    async with AsyncBaiduPanClient(client_id, client_secret, app_name) as client:
        quota = await client.get_quota()
        await client.upload(Path("file.txt"), file_to=Path("/docs/file.txt"))
        await client.download(dlink, file_to=Path("output.txt"))
"""

__version__ = "0.1.0"

from .client import AsyncBaiduPanClient
from .errors import (
    BaiduPanAPIError,
    BaiduPanError,
    BaiduPanNetworkError,
    TokenExpiredError,
)
from .models import ApiFileListItem, ApiFileMeta, ApiQuotaInfo, UploadResult

__all__ = [
    # Client
    "AsyncBaiduPanClient",
    # Errors
    "BaiduPanError",
    "BaiduPanNetworkError",
    "BaiduPanAPIError",
    "TokenExpiredError",
    # Models
    "UploadResult",
    "ApiFileMeta",
    "ApiFileListItem",
    "ApiQuotaInfo",
]



"""HTTP 传输层异步测试"""

from __future__ import annotations

import json
from unittest.mock import AsyncMock

import httpx
import pytest
import respx

from abdds._http import _HttpTransport, _truncate_data
from abdds.errors import BaiduPanAPIError, BaiduPanNetworkError, TokenExpiredError


class TestTruncateData:
    def test_short_string_unchanged(self):
        assert _truncate_data("hello") == "hello"

    def test_long_string_truncated(self):
        result = _truncate_data("x" * 1000)
        assert "truncated" in result
        assert len(result) < 1000

    def test_bytes_shown_as_binary(self):
        result = _truncate_data(b"\x00" * 100)
        assert "Binary Data" in result
        assert "100 bytes" in result

    def test_dict_values_truncated(self):
        result = _truncate_data({"key": "x" * 1000})
        assert "truncated" in result["key"]

    def test_list_items_truncated(self):
        result = _truncate_data(["x" * 1000])
        assert "truncated" in result[0]

    def test_nested_structure(self):
        data = {"a": [{"b": "x" * 1000}]}
        result = _truncate_data(data)
        assert "truncated" in result["a"][0]["b"]


class TestHttpTransportInit:
    def test_client_created(self):
        transport = _HttpTransport()
        assert transport.client is not None

    def test_custom_params(self):
        transport = _HttpTransport(max_retries=5, timeout=(10, 30))
        assert transport._max_retries == 5
        assert transport._timeout == (10, 30)


class TestHttpTransportRequest:
    @respx.mock
    async def test_successful_get(self):
        respx.get("https://example.com/api").mock(
            return_value=httpx.Response(200, json={"errno": 0, "data": "ok"})
        )

        transport = _HttpTransport(max_retries=0)
        transport.set_access_token("test_token")
        result = await transport.request("GET", "https://example.com/api")
        assert result["data"] == "ok"

    async def test_no_token_raises(self):
        transport = _HttpTransport(max_retries=0)
        with pytest.raises(TokenExpiredError):
            await transport.request("GET", "https://example.com/api")

    @respx.mock
    async def test_api_error_raised(self):
        respx.get("https://example.com/api").mock(
            return_value=httpx.Response(
                200,
                json={"errno": 31023, "errmsg": "file not found", "request_id": "req1"},
            )
        )

        transport = _HttpTransport(max_retries=0)
        transport.set_access_token("test_token")

        with pytest.raises(BaiduPanAPIError) as exc_info:
            await transport.request("GET", "https://example.com/api")

        assert exc_info.value.errno == 31023
        assert exc_info.value.errmsg == "file not found"
        assert exc_info.value.request_id == "req1"

    @respx.mock
    async def test_network_error_raised(self):
        respx.get("https://example.com/api").mock(
            side_effect=httpx.ConnectError("connection lost")
        )

        transport = _HttpTransport(max_retries=0)
        transport.set_access_token("test_token")

        with pytest.raises(BaiduPanNetworkError):
            await transport.request("GET", "https://example.com/api")

    @respx.mock
    async def test_token_expired_auto_refresh(self):
        # 第一次请求返回 token 过期
        route = respx.get("https://example.com/api")
        route.side_effect = [
            httpx.Response(200, json={"errno": -6, "errmsg": "token expired"}),
            httpx.Response(200, json={"errno": 0, "data": "refreshed"}),
        ]

        transport = _HttpTransport(max_retries=0)
        transport.set_access_token("old_token")

        # 设置异步 token 过期回调
        callback = AsyncMock(return_value="new_token")
        transport.set_token_expired_callback(callback)

        result = await transport.request("GET", "https://example.com/api")
        assert result["data"] == "refreshed"
        callback.assert_called_once()

    @respx.mock
    async def test_token_expired_callback_fails(self):
        respx.get("https://example.com/api").mock(
            return_value=httpx.Response(200, json={"errno": -6, "errmsg": "token expired"})
        )

        transport = _HttpTransport(max_retries=0)
        transport.set_access_token("old_token")

        # 异步回调返回 None（刷新失败）
        callback = AsyncMock(return_value=None)
        transport.set_token_expired_callback(callback)

        with pytest.raises(BaiduPanAPIError) as exc_info:
            await transport.request("GET", "https://example.com/api")
        assert exc_info.value.errno == -6

    @respx.mock
    async def test_invalid_json_response(self):
        respx.get("https://example.com/api").mock(
            return_value=httpx.Response(200, text="not json", headers={"content-type": "text/plain"})
        )

        transport = _HttpTransport(max_retries=0)
        transport.set_access_token("test_token")

        with pytest.raises(BaiduPanAPIError) as exc_info:
            await transport.request("GET", "https://example.com/api")
        assert exc_info.value.errno == -1


class TestHttpTransportClose:
    async def test_close(self):
        transport = _HttpTransport()
        # 先触发 client 初始化
        _ = transport.client
        await transport.close()
        assert transport._client is None
        assert transport._closed is True

    async def test_request_after_close_raises(self):
        """close() 后调用 request() 应抛 RuntimeError"""
        transport = _HttpTransport()
        transport.set_access_token("test_token")
        await transport.close()

        with pytest.raises(RuntimeError, match="Transport is closed"):
            await transport.request("GET", "https://example.com/api")



"""数据模型测试"""

from abdds.models import (
    ApiFileListItem,
    ApiFileMeta,
    _ApiPreUploadResponse,
    ApiQuotaInfo,
    UploadResult,
)


class TestUploadResult:
    def test_construction(self):
        r = UploadResult(fs_id=123, md5="abc", path="/apps/test/f.txt", size=1024)
        assert r.fs_id == 123
        assert r.md5 == "abc"
        assert r.path == "/apps/test/f.txt"
        assert r.size == 1024

    def test_equality(self):
        a = UploadResult(fs_id=1, md5="a", path="/a", size=10)
        b = UploadResult(fs_id=1, md5="a", path="/a", size=10)
        assert a == b


class TestApiQuotaInfo:
    def test_construction(self):
        q = ApiQuotaInfo(total=100, used=30)
        assert q.total == 100
        assert q.used == 30


class TestApiFileMeta:
    def test_construction(self):
        m = ApiFileMeta(md5="md5val", path="/apps/test/f.txt", size=50, fs_id=1, dlink="https://dlink", filename="f.txt")
        assert m.filename == "f.txt"
        assert m.dlink == "https://dlink"


class TestApiFileListItem:
    def test_construction(self):
        item = ApiFileListItem(md5="md5val", path="/apps/test/d", size=0, fs_id=2, is_dir=1, filename="d")
        assert item.is_dir == 1


class Test_ApiPreUploadResponse:
    def test_construction(self):
        r = _ApiPreUploadResponse(path="/a", uploadid="uid", block_list=[0], block_list_md5=["md5"], file_size=100)
        assert r.uploadid == "uid"
        assert r.block_list == [0]
